const create = (tagName, className, content) => {
  const element = document.createElement(tagName);
  className && (element.className = className);
  content && (element.innerHTML = content);
  return element;
};
const toggleInput = (terminal, enable = false) => {
  terminal.input.readOnly = !enable;
  terminal.inputContainer.style.opacity = enable ? "" : "0";
};
const buildTree = (node, prompt) => {
  node.className = "terminal";
  const commandContainer = create("div", "terminal-container");
  const inputContainer = create("div", "terminal-type");
  const promptContainer = create("span", void 0, prompt);
  const input = create("input");
  input.setAttribute("type", "text");
  inputContainer.append(promptContainer);
  inputContainer.append(input);
  node.append(commandContainer);
  node.append(inputContainer);
  node.addEventListener("click", () => input.focus());
  return { commandContainer, input, inputContainer };
};
const print = (content, isCommand, scrollIntoView, commandContainer, input, settings) => {
  const line = create("p", void 0, isCommand ? settings.prompt : content);
  if (isCommand) {
    const cmd = create("span", "terminal-command", content);
    line.append(cmd);
  }
  commandContainer.append(line);
  if (scrollIntoView) {
    input.scrollIntoView();
  }
};
var TerminalEvent = /* @__PURE__ */ ((TerminalEvent2) => {
  TerminalEvent2["ON_COMMAND"] = "onCommand";
  TerminalEvent2["ON_COMMAND_NOT_FOUND"] = "onCommand404";
  TerminalEvent2["ON_PROCESS_START"] = "onProcessStart";
  TerminalEvent2["ON_PROCESS_END"] = "onProcessStop";
  TerminalEvent2["ON_PROCESS_INTERRUPT"] = "onProcessInterrupt";
  TerminalEvent2["ON_INIT"] = "onInit";
  return TerminalEvent2;
})(TerminalEvent || {});
const dispatchEvent = (event, host, info) => {
  const customEvent = new CustomEvent(event, { detail: info });
  host.dispatchEvent(customEvent);
};
const evalCommand = (cmd, instance) => {
  const { print: print2 } = instance;
  const [command, ...args] = cmd.split(" ");
  const argMatches = args.join(" ").match(/('[^']+'|"[^"]+"|[^\s'"]+)/g);
  const commandArguments = argMatches === null ? [] : argMatches.map((element) => element.replace(/(^['"]|['"]$)/g, ""));
  const commandInstance = instance.settings.commands[command];
  if (commandInstance) {
    if (commandInstance.argDescriptions && commandInstance.argDescriptions.length > 0 && commandArguments.length === 0) {
      print2(`Usage: ${command} ${commandInstance.argDescriptions.map((arg) => `[${arg}]`).join(" ")}`);
    } else {
      commandInstance.func(instance, ...commandArguments);
      dispatchEvent(TerminalEvent.ON_COMMAND, instance.settings.host, cmd);
    }
  } else {
    print2(`<span class="terminal-error">command not found: ${command}</span>`);
    dispatchEvent(TerminalEvent.ON_COMMAND_NOT_FOUND, instance.settings.host, cmd);
  }
};
const startProcess = (terminal) => {
  const { settings: { host } } = terminal;
  toggleInput(terminal);
  terminal.isProcessRunning = true;
  dispatchEvent(TerminalEvent.ON_PROCESS_START, host);
};
const stopProcess = (terminal) => {
  const { input, settings: { host } } = terminal;
  toggleInput(terminal, true);
  terminal.isProcessRunning = false;
  input.focus();
  dispatchEvent(TerminalEvent.ON_PROCESS_END, host);
};
const typeText = (text, speed, terminal, isCommand) => new Promise((resolve) => {
  startProcess(terminal);
  const line = create("p", void 0, isCommand ? terminal.settings.prompt : "");
  const cmd = create("span", "terminal-command", "");
  if (isCommand) {
    line.append(cmd);
  }
  terminal.commandContainer.append(line);
  let i = 0;
  const typeTarget = isCommand ? cmd : line;
  const type = () => {
    if (i < text.length && terminal.isProcessRunning) {
      typeTarget.innerHTML += text.charAt(i);
      i++;
      setTimeout(type, speed);
    } else {
      stopProcess(terminal);
      resolve(i === text.length);
    }
  };
  setTimeout(type, speed);
});
const populateHistory = (instance) => {
  const { input: { value }, history, settings: { historyLength } } = instance;
  if (value === history[0]) {
    return;
  }
  if (history.length >= historyLength) {
    instance.history = [value, ...history.slice(0, -1)];
    return;
  }
  instance.history = [value, ...history];
  instance.lastHistoryIndex = 0;
};
const searchHistory = (instance, isDown) => {
  const { history, lastHistoryIndex } = instance;
  const endOfHistory = history.length - 1;
  let newIndex;
  if (isDown && lastHistoryIndex === 0)
    return;
  if (!isDown && lastHistoryIndex === history.length)
    return;
  if (endOfHistory < 0)
    return;
  if (isDown) {
    newIndex = lastHistoryIndex - 1;
    instance.input.value = newIndex - 1 >= 0 ? history[newIndex - 1] : "";
  } else {
    newIndex = lastHistoryIndex + 1;
    instance.input.value = history[lastHistoryIndex];
  }
  instance.lastHistoryIndex = newIndex;
};
const attachKeyboardListener = (host, instance) => {
  const { input, print: print2 } = instance;
  host.addEventListener("keyup", ({ key, ctrlKey }) => {
    if (ctrlKey && key === "c" && instance.isProcessRunning) {
      print2("^C");
      stopProcess(instance);
      dispatchEvent(TerminalEvent.ON_PROCESS_INTERRUPT, host);
    } else {
      if (instance.isProcessRunning)
        return;
      if (key === "Enter") {
        instance.lastHistoryIndex = 0;
        if (input.value.length > 0) {
          print2(input.value, true);
          populateHistory(instance);
          evalCommand(input.value, instance);
        } else {
          print2(" ", true);
        }
        input.value = "";
        return;
      }
      if (key === "ArrowUp") {
        searchHistory(instance);
        return;
      }
      if (key === "ArrowDown") {
        searchHistory(instance, true);
      }
    }
  });
};
const createHelp = (terminal) => ({
  name: "help",
  description: "shows a full list of all available commands",
  func: ({ print: print2 }) => {
    for (const cmd in terminal.settings.commands) {
      if (Object.hasOwnProperty.call(terminal.settings.commands, cmd)) {
        print2(terminal.settings.commands[cmd].name + " - " + terminal.settings.commands[cmd].description);
      }
    }
  }
});
const TERMINAL_STYLE = '.terminal{overflow-y:auto;overflow-x:hidden;background-color:var(--terminal-bg-color, black);padding:15px;box-sizing:border-box}.terminal *{color:var(--terminal-fg-color, white);font-family:var(--terminal-font, "Courier New", monospace)}.terminal-type{width:100%;display:flex}.terminal-type>span{padding-right:7pt}.terminal-type>input{flex-grow:2;background:transparent;border:0;font-size:inherit;padding:0}.terminal-type>input:focus{border:none;outline:none}.terminal-container>p{margin:0}span.terminal-command{color:var(--terminal-accent-color, #ffff7d)}span.terminal-error{color:var(--terminal-error-color, #cc1010)}';
const loadStyle = () => {
  if (document.head.querySelectorAll('link[data-type="terminal"]').length === 0) {
    const element = create("style", void 0, TERMINAL_STYLE);
    element.setAttribute("data-type", "terminal");
    element.setAttribute("type", "text/css");
    document.head.append(element);
  }
};
const setPrompt = (prompt, inputContainer, settings) => {
  settings.prompt = prompt;
  const promptContainer = inputContainer.querySelector("span");
  promptContainer.innerHTML = prompt;
};
const initTerminal = ({
  host,
  welcomeMessage,
  prompt = "$: ",
  historyLength = 50,
  history = [],
  enableHelp = true,
  commands
}) => {
  const settings = {
    host,
    welcomeMessage,
    prompt,
    historyLength,
    enableHelp,
    commands
  };
  loadStyle();
  const { commandContainer, input, inputContainer } = buildTree(host, prompt);
  const terminal = {
    history,
    lastHistoryIndex: 0,
    isProcessRunning: false,
    settings,
    commandContainer,
    inputContainer,
    input,
    print: (content, isCommand = false, scrollIntoView = true) => print(content, isCommand, scrollIntoView, commandContainer, input, settings),
    run: (cmd) => evalCommand(cmd, terminal),
    start: () => startProcess(terminal),
    stop: () => stopProcess(terminal),
    type: (text, speed = 60, isCommand) => typeText(text, speed, terminal, isCommand),
    setPrompt: (newPrompt) => setPrompt(newPrompt, inputContainer, settings)
  };
  if (enableHelp) {
    terminal.settings.commands.help = createHelp(terminal);
  }
  attachKeyboardListener(host, terminal);
  dispatchEvent(TerminalEvent.ON_INIT, host);
  welcomeMessage && terminal.print(welcomeMessage);
  return terminal;
};
export { TerminalEvent, initTerminal };
